#ifndef VEC_H
#define VEC_H
#include <stdbool.h>
#include <stddef.h>
/* Struct Declaration. The actual struct definition is hidden in `vec.c`.
 */
struct Vec;

/* Allows us to write `Vec` instead of `struct Vec`. */
typedef struct Vec Vec;

/* Returns a pointer to a freshly allocated vector of length 0 and capacity 1.
 * Returns NULL if out of memory.
 */
Vec* vec_new();

/* Free the dynamically allocated memory areas of `xs`. */
void vec_free(Vec* xs);

/* Returns a pointer to the element at index `i` of vector `xs`. */
void** vec_at(Vec* xs, size_t i);

/* Add a single element `x` to the end of the vector `xs`.
 * Returns `false` if out of memory, otherwise `true`.
 */
bool vec_push(Vec* xs, void* x);

/* Remove the element `i` of vector `xs` if it exists. Otherwise do nothing. */
void vec_remove(Vec* xs, size_t i);

/* Returns the number of elements currently stored in `xs`. */
size_t vec_length(Vec* xs);

/* Returns the vector's current capacity. */
size_t vec_capacity(Vec* xs);

/* Print the address, length, capacity, and elements of `xs`. */
void vec_print(Vec* xs);

#endif /* VEC_H */
